#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhumenu.h"

char *plugin_name = "HTML";
int  counter = 0;

static void print (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "\xC2\xA0");
		return;
	}
	while (*s != '\0') {
		if (*s == ' ') {
			fprintf(f, "\xC2\xA0");
			s++;
			continue;
		}
		if (*s == '"')
			fputc('\\', f);
		fputc(*s++, f);
	}
}

static void dump (FILE *f, const struct menu *menu, const char *lang, int n, const char *name, int offset)
{
	int i;
	fprintf(f, "z = new Array();\n");
	fprintf(f, "t = new Array();\n");
	fprintf(f, "l = new Array();\n");
	fprintf(f, "e = new Array();\n");
	for (i = 0; i < menu->nr; i++) {
		fprintf(f, "t[%d] = \"", i);	
		print(f, get_value(&menu->items[i], NULL, "Name", lang));
		fprintf(f, "\";\n");
		fprintf(f, "l[%d] = \"%s\";\n", i, menu->items[i].file_name);
		fprintf(f, "e[%d] = \"", i);	
		print(f, get_value(&menu->items[i], NULL, "Exec", lang));
		fprintf(f, "\";\n");
	}
	fprintf(f, "z[\"text\"] = t;\n");	
	fprintf(f, "z[\"load\"] = l;\n");	
	fprintf(f, "z[\"exec\"] = e;\n");	
	fprintf(f, "z[\"depth\"] = %d;\n", n);	
	fprintf(f, "z[\"offset\"] = %d;\n", offset);	
	fprintf(f, "data[\"%s\"] = z;\n", name);
	fprintf(f, "\n");
	for (i = 0; i < menu->nr; i++) {
		if (menu->items[i].issubmenu) {
			dump(f, menu->items[i].submenu, lang, n+1, menu->items[i].file_name, offset+i);
		}
	}
}

int write_menu_lang (const struct menu *menu, const char *path,
  const struct language *language)
{
	char *lang;
	char file[PATH_MAX];
	FILE *f;

	lang = language->name;
	if (lang[0] == '\0') lang = "C";
	sprintf(file, "%s/%s.html", path, lang);
	if ((f = fopen(file, "w")) == NULL) {
		fprintf(stderr, "html: fopen %s: %s\n", file, strerror(errno));
		return -1;
	}
	if (menu->nr > 0) {
		fprintf(f,
"<html>\n"
"<head>\n"
"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n"
"<script>\n"
"var locolor = \"#00ccff\";\n"
"var hicolor = \"#ffcc00\";\n"
"var hi   = new Array();\n"
"var list = new Array();\n"
"var data = new Array();\n"
"\n"
);
		dump(f, menu->items[0].submenu, lang, 0, "/", 0);
		fprintf(f,
"function func(f) {\n"
"	return ((navigator.appVersion.toLowerCase().indexOf('msie') >= 0) ? new Function(f) : f);\n"
"}\n"
"\n"
"function _remove(o) {\n"
"	o.removeChild(o.firstChild);\n"
"}\n"
"\n"
"function _replacechild(o, c) {\n"
"	o.replaceChild(c, o.firstChild);\n"
"}\n"
"\n"
"function setlo(o) {\n"
"	o.setAttribute(\"bgColor\", locolor);\n"
"}\n"
"\n"
"function sethi(o) {\n"
"	o.setAttribute(\"bgColor\", hicolor);\n"
"}\n"
"\n"
"function show(name, depth, exec) {\n"
"	_replacechild(document.getElementById(\"Exec\"), document.createTextNode(exec));\n"
"	for(i = depth; i < list.length; i++) {\n"
"		if (hi[i]) {\n"
"			for(j = 0; j < hi[i].length; j++)\n"
"				setlo(hi[i][j]);\n"
"			hi[i] = \"\";\n"
"		}\n"
"	}\n"
"	if (document.getElementById(name)) {\n"
"		hi[depth] = new Array();\n"
"		sethi((hi[depth][0] = document.getElementById(name)));\n"
"		sethi((hi[depth][1] = document.getElementById(name + \"/\")));\n"
"	}\n"
"	for(i = depth+1; i < list.length; i++) {\n"
"		if (list[i]) {\n"
"			list[i].style.display = \"none\";\n"
"			_remove(list[i]);\n"
"			list[i] = \"\";\n"
"		}\n"
"	}\n"
"	if (!data[name])\n"
"		return;\n"
"	_table = document.createElement(\"table\");\n"
"	_tbody = document.createElement(\"tbody\");\n"
"	for(i = 0; i < data[name][\"offset\"]; i++) {\n"
"		_td = document.createElement(\"td\");\n"
"		_td.appendChild(document.createTextNode(\"\\u00A0\"));\n"
"		_tr = document.createElement(\"tr\");\n"
"		_tr.appendChild(_td);\n"
"		_tbody.appendChild(_tr);\n"
"	}\n"
"	for(i = 0; i < data[name][\"text\"].length; i++) {\n"
"		_td = document.createElement(\"td\");\n"
"		_td.appendChild(document.createTextNode(data[name][\"text\"][i]));\n"
"		_td.setAttribute(\"id\", data[name][\"load\"][i]);\n"
"		_td.setAttribute(\"alt\", data[name][\"load\"][i]);\n"
"		_td.setAttribute(\"nowrap\", \"true\");\n"
"		_td.setAttribute(\"bgColor\", locolor);\n"
"		_td.setAttribute(\"onmouseover\", func(\"show('\"+data[name][\"load\"][i]+\"', \"+data[name][\"depth\"]+\", '\"+data[name][\"exec\"][i]+\"');\"));\n"
"		_tr = document.createElement(\"tr\");\n"
"		_tr.appendChild(_td);\n"
"		_td = document.createElement(\"td\");\n"
"		_td.appendChild(document.createTextNode(data[data[name][\"load\"][i]] ? \"\xC2\xA0\xC2\xBB\" : \"\"));\n"
"		_td.setAttribute(\"id\", data[name][\"load\"][i] + \"/\");\n"
"		_td.setAttribute(\"nowrap\", \"true\");\n"
"		_td.setAttribute(\"bgColor\", locolor);\n"
"		_td.setAttribute(\"onmouseover\", func(\"show('\"+data[name][\"load\"][i]+\"', \"+data[name][\"depth\"]+\", '\"+data[name][\"exec\"][i]+\"');\"));\n"
"		_tr.appendChild(_td);\n"
"		_tbody.appendChild(_tr);\n"
"	}\n"
"	_table.appendChild(_tbody);\n"
"	td = document.createElement(\"td\");\n"
"	td.appendChild(_table)\n"
"	_table.setAttribute(\"hspace\", \"0\");\n"
"	_table.setAttribute(\"vspace\", \"0\");\n"
"	_table.setAttribute(\"cellpadding\", \"0\");\n"
"	_table.setAttribute(\"cellspacing\", \"0\");\n"
"	_table.setAttribute(\"border\", \"0\");\n"
"	document.getElementById(\"_\").appendChild(td);\n"
"	list[data[name][\"depth\"]] = td;\n"
"}\n"
"function start() {\n"
"	show(\"/\", 0, \"\xC2\xA0\");\n"
"}\n"
"</script>\n"
"</head>\n"
"<body onload=\"start()\">\n"
"<noscript>Ehhez sajnos javascript kell!</noscript>\n"
"<tt><table border=\"0\" cellpadding=\"2\" cellspacing=\"0\" hspace=\"0\" vspace=\"0\" width=\"100%%\"><tr valign=\"top\"><td id=\"Exec\" bgcolor=\"#dddddd\">&nbsp;</td></tr></table></tt>\n"
"<table border=\"0\" cellpadding=\"2\" cellspacing=\"0\" hspace=\"0\" vspace=\"0\"><tr id=\"_\" valign=\"top\"></tr></table>\n"
"</body>\n"
"</html>\n"
);
	}
	fclose(f);
	return 0;
}

